import asyncio
import configparser
import os
import time
from concurrent.futures import ThreadPoolExecutor
from io import BytesIO
from typing import Optional, Union

import httpx
from PIL import Image, ImageDraw, ImageFont
from pydantic import AnyUrl

from .DataBaseManager import Data
from .DynamicChecker.DescChecker import UserProfilePendant


class HeadPicRender:
    def __init__(self,
                 uid: Optional[int] = None,
                 timestamp: Optional[int] = None,
                 face: Optional[AnyUrl] = None,
                 pendant: Union[UserProfilePendant, str, None] = None,
                 uname: Optional[str] = None,
                 official_verify: Optional[int] = None):

        self.__config = None
        self.__draw = None
        self.__container = None
        self.__uid = uid
        self.__timestamp = timestamp
        self.__face = face + "@240w_240h_1c_1s.webp"
        self.__pendant = pendant
        self.__uname = uname
        self.__base_path = os.path.dirname(os.path.abspath(__file__))
        self.__Font_path = os.path.join(self.__base_path, "Static", "Font")
        self.__Picture_path = os.path.join(self.__base_path, "Static", "Picture")
        self.__pendant_cache_path = os.path.join(self.__base_path, "Cache", "Pendant")
        self.__face_cache_path = os.path.join(self.__base_path, "Cache", "Face")
        self.__official_varify = official_verify
        self.__conf = configparser.ConfigParser()

    async def render_main_card_header(self):
        """
        渲染发送主动态的图片头
        :return:
        """
        render_info_list = []
        url_list = []
        # 读取配置文件
        self.__conf.read(os.path.join(self.__base_path, "config.ini"))
        # 读取背景色配置
        main_container_color = self.__conf.get("MainContainer", "MainContainerColor")
        # 读取昵称所用字体
        uname_font = self.__conf.get("main_header", "uname_font")
        # 读取昵称字体尺寸
        uname_font_size = self.__conf.getint("main_header", "uname_font_size")
        # 读取昵称字体颜色
        uname_color = self.__conf.get("main_header", "uname_color")

        # 读取时间字体
        time_font = self.__conf.get("main_header", "time_font")
        # 读取时间字体颜色
        time_color = self.__conf.get("main_header", "time_color")
        # 读取时间字体尺寸
        time_font_size = self.__conf.getint("main_header", "time_font_size")

        self.__container = Image.new("RGBA", (1108, 300), main_container_color)
        self.__draw = ImageDraw.Draw(self.__container)
        # 将时间戳格式化
        time_format = await self.__format_time(self.__timestamp)
        # 将昵称添加到渲染信息列表内
        render_info_list.append(
            {"type": "text", "font_color": uname_color,
             "font_size": uname_font_size,
             "content": self.__uname,
             "font": uname_font,
             "position": (164, 180)})

        # 将时间信息添加到渲染信息列表内
        render_info_list.append(
            {"type": "text", "font_color": time_color,
             "font": time_font,
             "font_size": time_font_size,
             "content": time_format, "position": (164, 240)})

        # 将头像的链接放入待请求的列表内
        url_list.append({"type": "face", "url": self.__face})
        # 将挂件信息放入请求列表中
        if self.__pendant and self.__pendant.image:
            url_list.append({"type": "pendant", "url": self.__pendant.image,
                             "path": os.path.join(self.__pendant_cache_path, self.__pendant.name + ".png")})

        # 将bili图标添加到渲染信息列表内
        bili_pic = Image.open(os.path.join(self.__Picture_path, "bilibili.png")).convert("RGBA").resize((154, 70))
        render_info_list.append({"type": "img", "content": bili_pic, "position": (477, 50)})
        # 请求图片，因为头像和挂件要按照图层的顺序粘贴，所以要map一下result
        zhaozhao = Image.open(os.path.join(self.__Picture_path, "zz.png")).resize((130, 130))
        render_info_list.append({"type": "img", "content": zhaozhao, "position": (900, 150)})

        with ThreadPoolExecutor(max_workers=2) as pool:
            results = pool.map(self.__get_images, url_list)
        for result in results:
            render_info_list.append(result)
        # 如果有官方认证的话
        if self.__official_varify != -1:
            official_verify_path = [os.path.join(self.__Picture_path, "official_yellow.png"),
                                    os.path.join(self.__Picture_path, "official_blue.png")]
            img = Image.open(official_verify_path[self.__official_varify]).resize((32, 32))
            render_info_list.append({"type": "img", "content": img, "position": (100, 240)})

        for i in render_info_list:
            await self.assembly_header(i)

        return self.__container

    async def render_origin_card_header(self):
        """
        渲染源动态头
        :return:
        """

        render_info_list = []
        url_list = [{"type": "face", "url": self.__face}]
        # 读取配置文件
        self.__conf.read(os.path.join(self.__base_path, "config.ini"))
        # 读取背景色
        main_container_color = self.__conf.get("RepostMainContainer", "MainContainerColor")
        # 读取渲染昵称所用字体
        uname_font = self.__conf.get("RepostHeader", "uname_font")
        # 读取渲染昵称所用的字体颜色
        uname_color = self.__conf.get("RepostHeader", "uname_color")
        # 读取字体尺寸
        uname_font_size = self.__conf.getint("RepostHeader", "uname_font_size")

        # 获取头像
        with ThreadPoolExecutor(max_workers=1) as pool:
            results = pool.map(self.__get_images, url_list)
        for i in results:
            # 将头像重设大小
            i["content"] = i["content"].resize((48, 48), Image.ANTIALIAS)
            # 重设头像的位置
            i["position"] = (10, 10)
            # 将头像信息放入渲染信息列表
            render_info_list.append(i)
        render_info_list.append({"type": "text", "font_color": uname_color,
                                 "font_size": uname_font_size,
                                 "content": self.__uname,
                                 "font": uname_font,
                                 "position": (70, 10)})
        self.__container = Image.new("RGBA", (1068, 60), main_container_color)
        self.__draw = ImageDraw.Draw(self.__container)
        for i in render_info_list:
            await self.assembly_header(i)

        return self.__container

    async def __format_time(self, timestamp: int) -> str:
        """
        格式化时间
        :param timestamp:int
        :return: str
        """
        return time.strftime("%Y-%m-%d %H:%M:%S", time.localtime(timestamp))

    def __get_images(self, img_info: dict):
        if img_info["type"] == "face":
            with Data() as data:
                result = data.acquire_uid(self.__uid)
                # 如果缓存的有头像
                if result:
                    # 如果缓存的时间和现在相差大于24小时
                    if time.time() - result[0] > 86400:
                        # 更新数据库时间戳并重新请求图片
                        data.update_timestamp(self.__uid, int(time.time()))
                        response = httpx.get(img_info["url"])
                        with open(os.path.join(self.__face_cache_path, str(self.__uid) + ".webp"), "wb") as f:
                            f.write(response.content)
                        img = Image.open(BytesIO(response.content)).convert("RGBA")
                    else:
                        # 如果缓存的时间和现在相差小于24小时，直接读取图片
                        try:
                            img = Image.open(os.path.join(self.__face_cache_path, str(self.__uid) + ".webp"))
                        # 如果图片被手动删掉了，可能读取不了，这个时候重新请求
                        except:
                            response = httpx.get(img_info["url"])
                            with open(os.path.join(self.__face_cache_path, str(self.__uid) + ".webp"), "wb") as f:
                                f.write(response.content)
                            img = Image.open(BytesIO(response.content)).convert("RGBA")
                # 如果没缓存过
                else:
                    # 请求图片
                    response = httpx.get(img_info["url"])
                    with open(os.path.join(self.__face_cache_path, str(self.__uid) + ".webp"), "wb") as f:
                        f.write(response.content)
                    # 写入时间和uid
                    data.add_uid(self.__uid, int(time.time()))
                    img = Image.open(BytesIO(response.content)).convert("RGBA")

            img = self.__make_face_circle(img)
            info = {"type": "img", "content": img, "position": (32, 172)}
        else:
            # 如果是pendant图片
            img_path = img_info["path"]
            # 如果缓存的有就直接读取
            if os.path.exists(img_path):
                img = Image.open(img_path).convert("RGBA").resize((164, 164))
            # 缓存的没有就发送网络请求
            else:
                response = httpx.get(img_info["url"])
                with open(img_path, "wb") as f:
                    f.write(response.content)
                img = Image.open(BytesIO(response.content)).convert("RGBA").resize((164, 164))
            info = {"type": "img", "content": img, "position": (0, 140)}
        return info

    async def assembly_header(self, document_info: dict) -> None:
        if document_info["type"] == "text":
            font = ImageFont.truetype(os.path.join(self.__Font_path, document_info["font"]), document_info["font_size"],
                                      encoding='utf-8')
            self.__draw.text(xy=document_info["position"], text=document_info["content"], font=font,
                             fill=document_info["font_color"])
        else:
            img = document_info["content"]
            self.__container.paste(img, document_info["position"], img)

    def __make_face_circle(self, avatar):
        """
        将头像裁剪成圆形
        :param avatar: 头像
        :return:处理好的头像
        """
        # 背景尺寸
        if avatar.size != (240, 240):
            avatar = avatar.resize((240, 240))
        bg_size = (240, 240)
        bg = Image.new('RGBA', bg_size, color=(255, 255, 255, 0))
        # 头像尺寸
        avatar_size = avatar.size
        # 头像路径
        # 新建一个蒙板图
        mask = Image.new('RGBA', avatar_size, color=(0, 0, 0, 0))
        # 画一个圆
        mask_draw = ImageDraw.Draw(mask)
        mask_draw.ellipse((0, 0, avatar_size[0], avatar_size[1]), fill=(0, 0, 0, 255))
        box = (0, 0, 240, 240)
        # 以下使用到paste(img, box=None, mask=None)方法
        #   img 为要粘贴的图片对你
        #   box 为图片 头像在 bg 中的位置
        #   mask 为蒙板，原理同 ps， 只显示 mask 中 Alpha 通道值大于等于1的部分
        bg.paste(avatar, box, mask)
        bg = bg.resize((96, 96), Image.ANTIALIAS)
        return bg
